﻿using System.Runtime.ConstrainedExecution;
using System.Runtime.Serialization;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using Autofac;
using FitConnect.BaseClasses;
using FitConnect.Models;
using FitConnect.Security;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace FitConnect;

public class Sender : FunctionalBaseClass {
    internal Sender(FitConnectEndpoints endpoints,
        X509Certificate2 certificate,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        ILogger? logger = null
    ) : base(endpoints,
        certificate, logger, oAuthService, submissionService, routeService) {
    }

    public static Sender Create(FitConnectEndpoints endpoints,
        X509Certificate2 certificate, ILogger? logger = null) {
        // TODO: Remove certificate from here.
        /*
new Sender(..., certificate) -> Nicht korrekt. Es gibt 1 Zertifikat muss pro Destination. Kann über die Submission API abgerufen werden (/v1/destinations/{destinationId}/keys/{keyId} mit keyId aus encryptionKid-Attribut in /v1/destinations/{destinationId})
         */

        var container = DiContainer.GetContainer(endpoints, certificate, null, logger);
        return container.Resolve<Sender>();
    }

    /// <summary>
    ///     Check public keys
    /// </summary>
    /// <returns></returns>
    public async Task<bool> CheckPublicKeyAsync(string publicKey) {
        throw new NotImplementedException();
    }

    public async Task<string> Encrypt(PublicKey publicKey, byte[] attachment) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     Create Metadata incl. Hash
    /// </summary>
    /// <param name="data"></param>
    /// <param name="attachment"></param>
    /// <returns></returns>
    public async Task<string>
        CreateMetadataAsync(string data, string attachment, string publicKey) {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Check public keys
    /// </summary>
    /// <returns></returns>
    public async Task<bool> CheckPublicKeyAsync() {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Encrypt Data
    /// </summary>
    /// <param name="data"></param>
    /// <returns></returns>
    public string Encrypt(PublicKey publicKey, string? data) {
        if (data == null) return "";
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Encrypt attachments (Anhänge)
    /// </summary>
    /// <param name="publicKey"></param>
    /// <param name="attachment"></param>
    /// <returns></returns>
    public KeyValuePair<string, string> Encrypt(PublicKey publicKey, Attachment attachment) {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Encrypt attachments (Anhänge)
    /// </summary>
    /// <param name="publicKey">Public key for encryption</param>
    /// <param name="attachments">List of attachments to encrypt</param>
    /// <returns></returns>
    public Dictionary<string, string>
        Encrypt(PublicKey publicKey, IEnumerable<Attachment> attachments) {
        return attachments.Select(a => Encrypt(publicKey, a))
            .ToDictionary(p => p.Key, p => p.Value);
    }

    /// <summary>
    ///     Create Metadata incl. Hash
    /// </summary>
    /// <param name="data"></param>
    /// <param name="attachment"></param>
    /// <returns></returns>
    public async Task<string> CreateMetadataAsync(string data, byte[] attachment) {
        throw new NotImplementedException();
    }

    public CreateSubmissionDto AddSubmission(Submission submission) {
        return new CreateSubmissionDto {
            DestinationId = submission.DestinationId,
            ServiceType = (ServiceTypeDto)submission.ServiceType,
            AnnouncedAttachments = submission.Attachments.Select(a => a.Id).ToList()
        };
    }

    /// <summary>
    ///     Uploading the encrypted data to the server
    /// </summary>
    /// <param name="encryptedAttachments">Encrypted attachments with id and content</param>
    /// <exception cref="NotImplementedException"></exception>
    public async Task<bool> UploadAttachmentsAsync(
        Dictionary<string, string> encryptedAttachments) {
        throw new NotImplementedException();
    }

    public async Task SubmitSubmissionAsync(Submission submission) {
        throw new NotImplementedException();
    }

    public Task<string> GetDestinationIdAsync(string leiaKey, string? ags, string? ars,
        string? areaId) {
        return RouteService.GetDestinationIdAsync(leiaKey, ags, ars, areaId);
    }


    /// <summary>
    ///     Encrypt Metadata with public key
    /// </summary>
    /// <param name="publicKey"></param>
    /// <param name="metadata"></param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public string Encrypt(PublicKey publicKey, Metadata metadata) {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Encrypt data with public key (Fachdaten)
    /// </summary>
    /// <param name="publicKey"></param>
    /// <param name="data"></param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public string Encrypt(PublicKey publicKey, Data data) {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Uploading the encrypted data and metadata to the server
    /// </summary>
    /// <param name="encryptedData"></param>
    /// <param name="encryptedMetadata"></param>
    /// <exception cref="NotImplementedException"></exception>
    public async Task<bool> SubmitAsync(string encryptedData, string encryptedMetadata) {
        throw new NotImplementedException();
    }

    private string Encrypt(object data) {
        var rsa = new RsaEncryption(Certificate, null);
        return Convert.ToBase64String(rsa.EncryptData(
            Encoding.UTF8.GetBytes(JsonConvert.SerializeObject(data))));
    }

    protected SubmitSubmissionDto CreateSubmitSubmissionDto(Submission submission) {
        var encryptData = Encrypt(submission.Data);
        var encryptMetadata = Encrypt(submission.Metadata);


        return new SubmitSubmissionDto {
            EncryptedData = encryptData,
            EncryptedMetadata = encryptMetadata,
        };
    }
}
