using System;
using System.Collections.Generic;
using Autofac;
using FitConnect;
using FitConnect.Models;
using FitConnect.Security;
using MockContainer;
using NUnit.Framework;

namespace FluentApiTest;

public class FluentSenderTests {
    private Client _client = null!;
    private string clientId = "clientId";
    private string clientSecret = "clientSecret";
    private IContainer _container = null!;

    [OneTimeSetUp]
    public void OneTimeSetup() {
        _container = Container.Create();
    }

    [SetUp]
    public void Setup() {
        var certificate = RsaEncryption.CreateSelfSignedCertificate();

        _client = new Client(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
            "", "", certificate, certificate, container: _container);
    }

    [Test]
    public void FluentSender_ShouldNotThrowAnError_MockingServicesWithData() {
        _client.Sender
            .WithDestination("destinationId")
            .WithAttachments(Array.Empty<Attachment>())
            .WithData("")
            .Submit();
    }

    [Test]
    public void FluentSender_ShouldNotThrowAnError_MockingServicesWithoutData() {
        _client.Sender
            .WithDestination("destinationId")
            .WithAttachments(Array.Empty<Attachment>())
            .Submit();
    }
}
